<?php

namespace Drupal\youtube_plus\Form;

use Drupal\Core\Url;
use Drupal\youtube_plus\YouTubePlusUtils;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Configuration of Channel entity.
 */
class ChannelForm extends EntityForm {

  /**
   * Indicates if is add or edit page.
   *
   * @var bool
   */
  protected $edit;

  /**
   * Google YouTube Service.
   *
   * @var \Google_Service_YouTube
   */
  protected $youtube;

  /**
   * Configuration settings.
   *
   * @var any
   */
  protected $config;

  /**
   * The channel to delete.
   *
   * @var string
   */
  protected $channel;
  /**
   * Drupal\Core\Plugin\DefaultPluginManager definition.
   *
   * @var Drupal\Core\Entity\Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityQuery;

  /**
   * Class Constructor.
   */
  public function __construct(EntityTypeManagerInterface $entityQuery) {
    $this->entityQuery = $entityQuery;
    $this->youtube = new YouTubePlusUtils();
    $this->config = \Drupal::service('config.factory')->getEditable('youtube_plus.settings');
  }

  /**
   * Class Create.
   *
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    return new static(
      // Load the service required to construct this class.
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {

    $form = parent::form($form, $form_state);
    if (empty($this->config->get('api_key'))) {
      \Drupal::messenger()->addMessage($this->t('You first need to define a API key <a href="@link">here</a>.', ["@link" => Url::fromRoute('youtube_plus.settings')->toSTring()]), 'warning');
    }

    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Channel Name'),
      '#required' => TRUE,
      '#default_value' => $this->entity->getName(),
    ];

    // @todo provide option to config by URL and get channel ID
    $form['channel_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Channel ID'),
      '#required' => FALSE,
      '#default_value' => $this->entity->getId(),
      '#states' => [
        'visible' => [
          ':input[name="type"]' => ['value' => 0],
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    $channel_id = $form_state->getValue('channel_id');
    $custom_url = $form_state->getValue('channel_custom_url');

    // No API key.
    if (empty($this->config->get('api_key'))) {
      $form_state->setErrorByName('info', $this->t('You must set API Key before adding a channel.'));

    }

    if (strpos($channel_id, ' ') !== FALSE || strpos($custom_url, ' ') !== FALSE) {
      $form_state->setErrorByName('info', $this->t('Fields must not contain spaces.'));
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $channel_id = $form_state->getValue('channel_id');
    $this->entity->set('id', $channel_id);
    $status = $this->entity->save();
    $this->entity->statusMessage($status);

    $form_state->setRedirectUrl(Url::fromRoute('entity.youtube_plus_channel.list'));

  }

}
